#!/usr/bin/env python
# encoding: utf-8
"""
copyright (c) 2016 Earth Advantage. All rights reserved.
..codeauthor::Paul Munday <paul@paulmunday.net>

Custom Exceptions/Errors
"""


class ConfigError(Exception):
    """Indicates an error when trying to obtain a config value."""
    pass



#!/usr/bin/env python
# encoding: utf-8
"""
copyright (c) 2016-2017 Earth Advantage.
All rights reserved
..codeauthor::Paul Munday <paul@paulmunday.net>
"""

# Local Imports
from yamlconf.config import Config
from yamlconf.exceptions import ConfigError



#!/usr/bin/env python
# encoding: utf-8
"""
copyright (c) 2016 Earth Advantage. All rights reserved.
..codeauthor::Paul Munday <paul@paulmunday.net>

Config class
"""
# pylint:disable=abstract-method

# Imports from Standard Library
import os
from copy import deepcopy

# Imports from Third Party Modules
import yaml

# Local Imports
from yamlconf.exceptions import ConfigError
from yamlconf.utils import alphasnake

BASE_PATH = os.getcwd()


def _suffix(name, suffix=None):
    """Append suffix (default _)."""
    suffix = suffix if suffix else '_'
    return "{}{}".format(name, suffix)


class Config(object):
    """Class for retrieving configuration variables.

    N.B. Recommended usage is to subclass this class to define default_file
    and/or default_config_root as necessary, or to override __init__ with
    relevant defaults.
    eg:
    class GBRConfig(Config):
        default_file = 'config.yaml'
        default_config_root = os.path.join(BASE_PATH, 'config')

        def __init__(self, config_file=None, config_dir=None, section=None):
            super(GBRConfig, self).__init__(
                config_file=config_file, config_dir=config_dir,
                section=section, env_prefix='GBR_CONFIG'
            )
    """
    # pylint: disable=too-few-public-methods, too-many-instance-attributes
    default_file = 'config.yaml'
    default_config_root = os.path.join(BASE_PATH, 'config')

    def __init__(self,
                 config_file=None, config_dir=None,
                 section=None, env_prefix=None):
        if not env_prefix:
            raise ConfigError('env_prefix can not be null.')
        self.env_prefix = env_prefix
        self.config = {}
        self.prefix = None
        self.config_path = self.env_prefix + '_PATH'
        self.config_prefix = self.env_prefix + '_PREFIX'
        self.config_root_path = self.env_prefix + '_ROOT'
        self.basepath = os.getenv(
            self.config_path, default=self.default_config_root
        )
        self.config_root = os.getenv(
            self.config_root_path, default=self.default_config_root
        )
        self.section = section
        self.config_file = self._get_filepath(
            filename=config_file, config_dir=config_dir
        )
        self.load()

    def load(self):
        """(Re)Load config file."""
        try:
            with open(self.config_file) as configfile:
                self.config = yaml.load(configfile)
        except TypeError:
            # no config file (use environment variables)
            pass
        if self.config:
            self.prefix = self.config.get('config_prefix', None)
        if not self.prefix:
            if os.getenv(self.config_prefix):
                self.prefix = os.getenv(self.config_prefix)
            else:
                for path in [
                    os.path.join(self.basepath, self.default_file),
                    os.path.join(self.config_root, self.default_file)
                ]:
                    if os.path.exists(path):
                        with open(path) as conf:
                            config = yaml.load(conf)
                            prefix = config.get(
                                self.config_prefix.lower(), None
                            )
                            if prefix:
                                self.prefix = prefix
                                break

    def get(self, var, section=None, **kwargs):
        """Retrieve a config var.
        Return environment variable if it exists
        (ie [self.prefix + _] + [section + _] + var)
        otherwise var from config file.

        If both are null and no default is set a ConfigError will be raised,
        otherwise default will be returned.

        :param var: key to look up
        :param default: default value, must be supplied as keyword
        """
        # default is not a specified keyword argument so we can distinguish
        # between a default set to None and no default sets
        if not section and self.section:
            section = self.section
        default = kwargs.get('default', None)
        env_var = "{}{}{}".format(
            _suffix(self.prefix) if self.prefix else '',
            _suffix(alphasnake(section)) if section else '',
            alphasnake(str(var))
        ).upper()
        config = self.config.get(section, {}) if section else self.config
        result = config.get(var, default)
        result = os.getenv(env_var, default=result)
        # no default keyword supplied (and no result)
        #  use is None to allow empty lists etc
        if result is None and 'default' not in kwargs:
            msg = "Could not find '{}'".format(var)
            if section:
                msg = "{} in section '{}'.".format(
                    msg, section
                )
            msg = "{} Checked environment variable: {}".format(msg, env_var)
            if self.config_file:
                msg = "{} and file: {}".format(msg, self.config_file)
            raise ConfigError(msg)
        return result

    def keys(self, section=None):
        """Provide dict like keys method"""
        if not section and self.section:
            section = self.section
        config = self.config.get(section, {}) if section else self.config
        return config.keys()

    def items(self, section=None):
        """Provide dict like items method"""
        if not section and self.section:
            section = self.section
        config = self.config.get(section, {}) if section else self.config
        return config.items()

    def values(self, section=None):
        """Provide dict like values method"""
        if not section and self.section:
            section = self.section
        config = self.config.get(section, {}) if section else self.config
        return config.values()

    def __copy__(self):
        raise NotImplementedError('Shallow copying is forbidden')

    def copy(self):
        """Raise error"""
        self.__copy__()

    def __deepcopy__(self, memo):
        if self.section:
            config = self.config.get(self.section, {})
        else:
            config = self.config
        return deepcopy(config, memo)

    def _get_filepath(self, filename=None, config_dir=None):
        """
        Get config file.

            :param filename: name of config file (not path)
            :param config_dir: dir name prepended to file name.

        Note: we use e.g. GBR_CONFIG_DIR here, this is the default
        value in GBR but it is actually self.env_prefix + '_DIR' etc.

        If config_dir is not supplied it will be set to the value of the
        environment variable GBR_CONFIG_DIR or None.

        If filename is not supplied and the environment variable GBR_CONFIG
        is set and contains a path, its value will be tested to see if a file
        exists, if so that is returned as the config file otherwise filename
        will be set to GBR_CONFIG, if it exists, otherwise 'config.yaml'.

        If a filename is supplied or GBR_CONFIG is not an existing file:

            If the environment variable GBR_CONFIG_PATH exists the path
            GBR_CONFIG_PATH/config_dir/filename is checked.

            If it doesn't exist config/CONFIG_DIR/filename  is checked
                (relative to the root of the (GBR) repo)
            finally GBR_CONFIG_DEFAULT/CONFIG_DIR/filename is tried

        If no file is found None will be returned.
        """
        # pylint: disable=no-self-use
        config_file = None
        config_dir_env_var = self.env_prefix + '_DIR'
        if not filename:
            # Check env vars for config
            filename = os.getenv(self.env_prefix, default=self.default_file)
            # contains path so try directly
            if os.path.dirname(filename) and os.path.exists(filename):
                config_file = filename
        if not config_file:
            # Cannot contain path
            filename = os.path.basename(filename)
            if not config_dir:
                config_dir = os.getenv(config_dir_env_var, default='')
            for path in [self.basepath, self.config_root]:
                filepath = os.path.join(path, config_dir, filename)
                if os.path.exists(filepath):
                    config_file = filepath
                    break
        return config_file



#!/usr/bin/env python
# encoding: utf-8
"""
copyright (c) 2016 Earth Advantage. All rights reserved.
..codeauthor::Paul Munday <paul@paulmunday.net>

Tests for Config store
"""

# Imports from Standard Library
import os
import sys
import unittest
from copy import deepcopy

# Local Imports
from yamlconf import Config, ConfigError

PY3 = sys.version_info[0] == 3
if PY3:
    from unittest import mock
else:
    import mock


BASE_PATH = os.getcwd()
TEST_CONFIG_ROOT = os.path.join(BASE_PATH, 'config')
# Constants


# Helper Functions & Classes
class TESTConfig(Config):
    """Config class for TEST"""
    # pylint: disable=too-few-public-methods
    default_file = 'config.yaml'
    default_config_root = TEST_CONFIG_ROOT

    def __init__(self, config_file=None, config_dir=None, section=None):
        super(TESTConfig, self).__init__(
            config_file=config_file, config_dir=config_dir, section=section,
            env_prefix='TEST_CONFIG'
        )


# Tests
class ConfigTests(unittest.TestCase):
    """Tests for config functionality."""
    # pylint:disable=protected-access, invalid-name

    def setUp(self):
        self.env_vars = {
            'TEST_CONFIG_PATH': '/var/lib/gbr/etc',
            'TEST_CONFIG_ROOT': '/etc/gbr/conf',
            'TEST_CONFIG_PREFIX': 'TEST',
        }

    def tearDown(self):
        keys = list(self.env_vars.keys())
        keys.append('TEST_CONFIG_DIR')
        for key in keys:
            if key in os.environ:
                del os.environ[key]

    def test_get_filepath(self):
        """Test _get_filepath method."""
        conf = TESTConfig()
        with mock.patch('yamlconf.config.os.path.exists') as mock_exists:
            # no filename, not found in basepath
            mock_exists.side_effect = [False, True]
            expected = os.path.join(TEST_CONFIG_ROOT, 'config.yaml')
            result = conf._get_filepath()
            self.assertEqual(expected, result)

            # with filename, not found in basepath
            mock_exists.side_effect = [False, True]
            expected = os.path.join(TEST_CONFIG_ROOT, 'test.yaml')
            result = conf._get_filepath(filename='test.yaml')
            self.assertEqual(expected, result)

            # no filename, found in basepath
            mock_exists.side_effect = [True, False]
            expected = os.path.join(conf.basepath, 'config.yaml')
            result = conf._get_filepath()
            self.assertEqual(expected, result)

            # with filename, found in basepath
            mock_exists.side_effect = [True, False]
            expected = os.path.join(conf.basepath, 'test.yaml')
            result = conf._get_filepath(filename='test.yaml')
            self.assertEqual(expected, result)

    def test_get_filepath_with_config_root(self):
        """Test _get_filepath method with TEST_CONFIG_ROOT env var set."""
        gbr_config_root = '/var/lib/gbr/'
        os.environ['TEST_CONFIG_ROOT'] = gbr_config_root
        conf = TESTConfig()
        with mock.patch('yamlconf.config.os.path.exists') as mock_exists:
            # TEST_CONFIG_ROOT env var set, no filename, not found in basepath
            mock_exists.side_effect = [False, True]
            expected = os.path.join(gbr_config_root, 'config.yaml')
            result = conf._get_filepath()
            self.assertEqual(expected, result)

            # TEST_CONFIG_ROOT env var set, with filename, not found in
            # basepath
            mock_exists.side_effect = [False, True]
            expected = os.path.join(gbr_config_root, 'test.yaml')
            result = conf._get_filepath(filename='test.yaml')
            self.assertEqual(expected, result)

            # TEST_CONFIG_ROOT env var set, no filename, found in basepath
            mock_exists.side_effect = [True, False]
            expected = os.path.join(conf.basepath, 'config.yaml')
            result = conf._get_filepath()
            self.assertEqual(expected, result)

            # TEST_CONFIG_ROOT env var set, with filename, found in basepath
            mock_exists.side_effect = [True, False]
            expected = os.path.join(conf.basepath, 'test.yaml')
            result = conf._get_filepath(filename='test.yaml')
            self.assertEqual(expected, result)

    def test_get_filepath_with_config_path(self):
        """Test _get_filepath method with TEST_CONFIG_PATH env var set."""
        gbr_config_path = '/var/lib/gbr/'
        os.environ['TEST_CONFIG_PATH'] = gbr_config_path
        conf = TESTConfig()
        with mock.patch('yamlconf.config.os.path.exists') as mock_exists:
            # TEST_CONFIG_PATH env var set, no filename, not found in basepath
            mock_exists.side_effect = [False, True]
            expected = os.path.join(TEST_CONFIG_ROOT, 'config.yaml')
            result = conf._get_filepath()
            self.assertEqual(expected, result)

            # TEST_CONFIG_PATH env var set, with filename, not found in
            # basepath
            mock_exists.side_effect = [False, True]
            expected = os.path.join(TEST_CONFIG_ROOT, 'test.yaml')
            result = conf._get_filepath(filename='test.yaml')
            self.assertEqual(expected, result)

            # TEST_CONFIG_PATH env var set, no filename, found in basepath
            mock_exists.side_effect = [True, False]
            expected = os.path.join(gbr_config_path, 'config.yaml')
            result = conf._get_filepath()
            self.assertEqual(expected, result)

            # TEST_CONFIG_PATH env var set, with filename, found in basepath
            mock_exists.side_effect = [True, False]
            expected = os.path.join(gbr_config_path, 'test.yaml')
            result = conf._get_filepath(filename='test.yaml')
            self.assertEqual(expected, result)

    def test_get_filepath_with_config_dir(self):
        """Test _get_filepath method with config_dir set."""
        conf = TESTConfig()
        config_dir = 'test'
        with mock.patch('yamlconf.config.os.path.exists') as mock_exists:
            # no filename, not found in basepath
            mock_exists.side_effect = [False, True]
            expected = os.path.join(
                TEST_CONFIG_ROOT, config_dir, 'config.yaml')
            result = conf._get_filepath(config_dir=config_dir)
            self.assertEqual(expected, result)

            # with filename, not found in basepath
            mock_exists.side_effect = [False, True]
            expected = os.path.join(TEST_CONFIG_ROOT, config_dir, 'test.yaml')
            result = conf._get_filepath(
                filename='test.yaml', config_dir=config_dir
            )
            self.assertEqual(expected, result)

            # no filename, found in basepath
            mock_exists.side_effect = [True, False]
            expected = os.path.join(conf.basepath, config_dir, 'config.yaml')
            result = conf._get_filepath(config_dir=config_dir)
            self.assertEqual(expected, result)

            # with filename, found in basepath
            mock_exists.side_effect = [True, False]
            expected = os.path.join(conf.basepath, config_dir, 'test.yaml')
            result = conf._get_filepath(
                filename='test.yaml', config_dir=config_dir
            )
            self.assertEqual(expected, result)

    @mock.patch('yamlconf.config.os.path.exists')
    @mock.patch('yamlconf.config.os.path.dirname')
    def test_get_filepath_with_gbr_config(self, mock_dirname, mock_exists):
        """Test _get_filepath method with TEST_CONFIG env var"""
        mock_dirname.side_effect = ['foo/bar/baz', 'foo/bar/', 'foo', '/etc/']
        mock_exists.side_effect = [True, False, False]
        gbr_config = '/etc/gbr.yaml'
        os.environ['TEST_CONFIG'] = gbr_config
        with mock.patch(
            'yamlconf.config.open', mock.mock_open(read_data='{}'),
            create=True
        ):
            conf = TESTConfig()
            expected = gbr_config
            self.assertEqual(expected, conf.config_file)

    def test_init_env_vars(self):
        """Test setting attributes via environment variables."""
        for key, val in self.env_vars.items():
            os.environ[key] = val
        conf = TESTConfig()
        self.assertEqual(self.env_vars['TEST_CONFIG_PATH'], conf.basepath)
        self.assertEqual(self.env_vars['TEST_CONFIG_ROOT'], conf.config_root)
        self.assertEqual(self.env_vars['TEST_CONFIG_PREFIX'], conf.prefix)

    @mock.patch('yamlconf.config.yaml.load')
    def test_init_config_file(self, mock_load):
        """Test setting attributes via config file."""
        mock_load.return_value = {'config_prefix': 'gbr_config_prefix'}
        with mock.patch(
            'yamlconf.config.open', mock.mock_open(read_data='{}'),
            create=True
        ):
            config_path = self.env_vars['TEST_CONFIG_PATH']
            config_root = self.env_vars['TEST_CONFIG_ROOT']
            for key, val in self.env_vars.items():
                os.environ[key] = val
            os.environ['TEST_CONFIG_DIR'] = 'config'
            conf = TESTConfig()
        self.assertEqual(config_path, conf.basepath)
        self.assertEqual(config_root, conf.config_root)
        # self.assertEqual(TEST_CONFIG_ROOT, conf.basepath)
        # self.assertEqual(TEST_CONFIG_ROOT, conf.config_root)
        self.assertEqual(None, conf.config_file)
        self.assertEqual('gbr_config_prefix', conf.prefix)

    @mock.patch('yamlconf.config.yaml.load')
    @mock.patch('yamlconf.config.os.path.exists')
    @mock.patch('yamlconf.config.os.path.dirname')
    def test_init_prefix(self, mock_dirname, mock_exists, mock_load):
        """Test setting attributes via config file."""
        mock_dirname.side_effect = ['foo/bar/baz', 'foo/bar/', 'foo', '']
        mock_exists.side_effect = [False, False, True, True]
        mock_load.side_effect = [{}, {'config_prefix': 'TEST'}]
        with mock.patch(
            'yamlconf.config.open', mock.mock_open(read_data='{}'),
            create=True
        ):
            for key, val in self.env_vars.items():
                os.environ[key] = val
            os.environ['TEST_CONFIG_PREFIX'] = 'TEST'
            conf = TESTConfig()
        self.assertEqual('TEST', conf.prefix)

    def test_get_env_var(self):
        """Test get from environment var"""
        expected = 'test'
        os.environ['TEST_CONFIG_PREFIX'] = 'TEST'
        os.environ['TEST_TEST_VALUE'] = expected
        conf = TESTConfig()
        result = conf.get('value', section='test')
        self.assertEqual(expected, result)

    def test_get_env_var_underscore(self):
        """Test get from environment var works with underscore in name"""
        expected = 'under_score'
        os.environ['TEST_CONFIG_PREFIX'] = 'TEST'
        os.environ['TEST_TEST_TEST_VALUE'] = expected
        conf = TESTConfig()
        result = conf.get('test_value', section='test')
        self.assertEqual(expected, result)

    def test_get_env_default(self):
        """Test get from default"""
        conf = TESTConfig()
        result = conf.get('value2', section='test', default='default')
        self.assertEqual('default', result)

    def test_get_exception(self):
        """Test get raises exception if not found"""
        os.environ['TEST_CONFIG_PREFIX'] = 'TEST'
        conf = TESTConfig()
        conf.config_file = None     # in case a local file exists
        with self.assertRaises(ConfigError) as conm:
            conf.get('foo !3', section='test')
        exception = conm.exception
        expected = "Could not find 'foo !3' in section 'test'. "
        expected += "Checked environment variable: TEST_TEST_FOO_3"
        self.assertEqual(expected, str(exception))

    @mock.patch('yamlconf.config.yaml.load')
    @mock.patch('yamlconf.config.os.path.exists')
    @mock.patch('yamlconf.config.os.path.dirname')
    def test_get_config(self, mock_dirname, mock_exists, mock_load):
        """Test get via config file."""
        mock_dirname.side_effect = ['foo/bar/baz', 'foo/bar/', 'foo', '']
        mock_exists.side_effect = [False, False, True]
        mock_load.side_effect = [
            {'test': {'test2': 'foo'}, 'config_prefix': 'TEST'}
        ]
        with mock.patch(
            'yamlconf.config.open', mock.mock_open(read_data='{}'),
            create=True
        ):
            conf = TESTConfig()
        result = conf.get('test2', section='test', default='default')
        self.assertEqual('foo', result)

    @mock.patch('yamlconf.config.yaml.load')
    @mock.patch('yamlconf.config.os.path.exists')
    @mock.patch('yamlconf.config.os.path.dirname')
    def test_copy(self, mock_dirname, mock_exists, mock_load):
        """Test copy raises error."""
        mock_dirname.side_effect = ['foo/bar/baz', 'foo/bar/', 'foo', '']
        mock_exists.side_effect = [False, False, True]
        mock_load.side_effect = [
            {'test': {'test2': 'foo'}, 'config_prefix': 'TEST'}
        ]
        with mock.patch(
            'yamlconf.config.open', mock.mock_open(read_data='{}'),
            create=True
        ):
            conf = TESTConfig()
        with self.assertRaises(NotImplementedError) as conm:
            conf.copy()
            msg = str(conm.exception)
            self.assertEqual('Shallow copying is forbidden.', msg)

    @mock.patch('yamlconf.config.yaml.load')
    @mock.patch('yamlconf.config.os.path.exists')
    @mock.patch('yamlconf.config.os.path.dirname')
    def test_deepcopy(self, mock_dirname, mock_exists, mock_load):
        """Test deepcopy."""
        mock_dirname.side_effect = ['foo/bar/baz', 'foo/bar/', 'foo', '']
        mock_exists.side_effect = [False, False, True]
        mock_load.side_effect = [
            {'test': {'test2': 'foo'}, 'config_prefix': 'TEST'}
        ]
        with mock.patch(
            'yamlconf.config.open', mock.mock_open(read_data='{}'),
            create=True
        ):
            conf = TESTConfig()
            new = deepcopy(conf)
            self.assertEqual(new, conf.config)
            new['foo'] = 'bar'
            self.assertNotEqual(new, conf.config)

    @mock.patch('yamlconf.config.yaml.load')
    @mock.patch('yamlconf.config.os.path.exists')
    @mock.patch('yamlconf.config.os.path.dirname')
    def test_items(self, mock_dirname, mock_exists, mock_load):
        """Test items."""
        mock_dirname.side_effect = ['foo/bar/baz', 'foo/bar/', 'foo', '']
        mock_exists.side_effect = [False, False, True]
        mock_load.side_effect = [
            {'test': {'test2': 'foo'}, 'config_prefix': 'TEST'}
        ]
        with mock.patch(
            'yamlconf.config.open', mock.mock_open(read_data='{}'),
            create=True
        ):
            conf = TESTConfig(section='test')
            self.assertEqual(conf.items(), conf.config['test'].items())

    @mock.patch('yamlconf.config.yaml.load')
    @mock.patch('yamlconf.config.os.path.exists')
    @mock.patch('yamlconf.config.os.path.dirname')
    def test_keys(self, mock_dirname, mock_exists, mock_load):
        """Test keys."""
        mock_dirname.side_effect = ['foo/bar/baz', 'foo/bar/', 'foo', '']
        mock_exists.side_effect = [False, False, True]
        mock_load.side_effect = [
            {'test': {'test2': 'foo'}, 'config_prefix': 'TEST'}
        ]
        with mock.patch(
            'yamlconf.config.open', mock.mock_open(read_data='{}'),
            create=True
        ):
            conf = TESTConfig(section='test')
            self.assertEqual(conf.keys(), conf.config['test'].keys())

    @mock.patch('yamlconf.config.yaml.load')
    @mock.patch('yamlconf.config.os.path.exists')
    @mock.patch('yamlconf.config.os.path.dirname')
    def test_values(self, mock_dirname, mock_exists, mock_load):
        """Test values."""
        mock_dirname.side_effect = ['foo/bar/baz', 'foo/bar/', 'foo', '']
        mock_exists.side_effect = [False, False, True]
        mock_load.side_effect = [
            {'test': {'test2': 'foo'}, 'config_prefix': 'TEST'}
        ]
        with mock.patch(
            'yamlconf.config.open', mock.mock_open(read_data='{}'),
            create=True
        ):
            conf = TESTConfig(section='test')
            self.assertEqual(
                list(conf.values()), list(conf.config['test'].values())
            )



#!/usr/bin/env python
# encoding: utf-8
"""
copyright (c) 2016 Earth Advantage. All rights reserved.
..codeauthor::Paul Munday <paul@paulmunday.net>,
              Fable Turas <fable@raintechpdx.com>

Utility functions for text translations etc.
"""

# Imports from Standard Library
import collections
import os
import re
import unicodedata
from datetime import date, datetime
from importlib import import_module
from typing import (  # noqa pylint: disable=unused-import
    Any,
    Mapping,
    Optional,
    Sequence,
)

# Constants
VALID_CHARS = re.compile(r'\W+')     # alphannumerics and underscore
STRIP_CHAR_CATS = (
    'M', 'S', 'C', 'Nl', 'No', 'Pc', 'Ps', 'Pe', 'Pi', 'Pf', 'Po'
)

STRIP_PUNC_CATS = ('Z', 'Pd')
STRIP_ALL_CATS = STRIP_CHAR_CATS + STRIP_PUNC_CATS


# Public Classes and Functions
def alphasnake(string):
    """Convert to snakecase removing non alpha numerics
    Word #word -> word_word.
    """
    if string:
        string = " ".join(
            [re.sub(r'\W+', '', word) for word in string.split()]
        )
        string = decamel_to_snake(string)
    return string


def decamel(string):
    """"Split CamelCased words.

    CamelCase -> Camel Case, dromedaryCase -> dromedary Case.
    """
    regex = re.compile(r'(\B[A-Z][a-z]*)')
    return regex.sub(r' \1', string)


def decamel_to_snake(string):
    """Convert to lower case, join camel case with underscore.
    CamelCase -> camel_case. Camel Case -> camel_case.
    """
    strings = [decamel(word) for word in string.split()]
    return "_".join([snake(dstring)for dstring in strings])


def snake(string):
    """Convert to snake case.
    Word word -> word_word
    """
    return "_".join([word.lower() for word in string.split()])
